/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.basic.provider;

import it.tidalwave.metadata.MetadataItemHolder;
import java.beans.PropertyVetoException;
import java.util.Date;
import java.io.IOException;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import static it.tidalwave.metadata.Metadata.StorageType.*;
import it.tidalwave.metadata.basic.Basic;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class BasicProviderTest 
  {
    private BasicProvider basicProvider;
    
    @Before
    public void setUp()
      throws PropertyVetoException, IOException 
      {
        basicProvider = new BasicProvider();
      }

    @After
    public void tearDown() 
      {
        basicProvider = null;
      }

    @Test
    public void testGetManagedClass() 
      {
        assertEquals(Basic.class, basicProvider.getItemClass());
      }  

    @Test
    public void testFindMetadataItemFromGoodFile()
      throws Exception
      {
        final DataObject dataObject = DataObject.find(FileUtil.createMemoryFileSystem().getRoot().createData("goodfile"));       
        final MetadataItemHolder<Basic> holder = basicProvider.findOrCreateItems(dataObject, EXTERNAL).iterator().next();
        assertNotNull(holder);
        assertTrue(holder.isAvailable()); // FIXME
      }
    
    @Test
    public void testFindMetadataItemFromBadJPG()
      throws Exception
      {
        final DataObject dataObject = DataObject.find(FileUtil.createMemoryFileSystem().getRoot().createData("badfile"));       
        final MetadataItemHolder<Basic> holder = basicProvider.findOrCreateItems(dataObject, EXTERNAL).iterator().next();
        assertNotNull(holder);
        assertFalse(holder.isAvailable());
      }
    
    @Test
    public void testLastestModificationTimeIsUpdated()
      throws Exception
      {
        final DataObject dataObject = DataObject.find(FileUtil.createMemoryFileSystem().getRoot().createData("goodfile"));       
        final MetadataItemHolder<Basic> holder = basicProvider.findOrCreateItems(dataObject, EXTERNAL).iterator().next();
        assertNotNull(holder);
        final Date timestamp1 = holder.getLatestModificationTime();
        assertTrue(holder.isAvailable());
        
        Thread.sleep(1000);
        final Date now = new Date();
        holder.get().setFileSize(10000L);
        final Date timestamp2 = holder.getLatestModificationTime();
        final long elapsed = timestamp2.getTime() - now.getTime();
        assertTrue("Too long: " + elapsed, elapsed < 50);
        assertNotSame(timestamp1, timestamp2);
      }
  }